<?php

namespace Cas\Services\Policies;

use Litepie\User\Contracts\UserPolicy;
use Cas\Services\Models\Services;

class ServicesPolicy
{

    /**
     * Determine if the given user can view the services.
     *
     * @param UserPolicy $user
     * @param Services $services
     *
     * @return bool
     */
    public function view(UserPolicy $user, Services $services)
    {
        if ($user->canDo('services.services.view') && $user->isAdmin()) {
            return true;
        }

        return $services->user_id == user_id() && $services->user_type == user_type();
    }

    /**
     * Determine if the given user can create a services.
     *
     * @param UserPolicy $user
     * @param Services $services
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('services.services.create');
    }

    /**
     * Determine if the given user can update the given services.
     *
     * @param UserPolicy $user
     * @param Services $services
     *
     * @return bool
     */
    public function update(UserPolicy $user, Services $services)
    {
        if ($user->canDo('services.services.edit') && $user->isAdmin()) {
            return true;
        }

        return $services->user_id == user_id() && $services->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given services.
     *
     * @param UserPolicy $user
     * @param Services $services
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Services $services)
    {
        return $services->user_id == user_id() && $services->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given services.
     *
     * @param UserPolicy $user
     * @param Services $services
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Services $services)
    {
        if ($user->canDo('services.services.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given services.
     *
     * @param UserPolicy $user
     * @param Services $services
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Services $services)
    {
        if ($user->canDo('services.services.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
